/*
 * The PKI console will be replaced first by feature parity in CLI utilities, then later by a new GUI.
 * This package will be removed once its functionality is replicated in the CLI utilities.
 * No further maintenance of this package will be performed.
 * Do not introduce new dependencies on this package; begin to remove existing dependencies.
 */
@Deprecated(since="10.14.0", forRemoval=true)
package com.netscape.admin.certsrv;