// --- BEGIN COPYRIGHT BLOCK ---
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//
// (C) 2007 Red Hat, Inc.
// All rights reserved.
// --- END COPYRIGHT BLOCK ---
package org.dogtagpki.legacy.core.policy;

import com.netscape.cmscore.util.AssertionException;

/**
 * Class to keep track of various configurable implementations.
 */
class RegisteredPolicy {
    String mId;
    String mClPath;

    public RegisteredPolicy(String id, String clPath) {
        if (id == null || clPath == null)
            throw new AssertionException("Policy id or classpath can't be null");
        mId = id;
        mClPath = clPath;
    }

    public String getClassPath() {
        return mClPath;
    }

    public String getId() {
        return mId;
    }
}
